<?php
namespace Datec\DatecFeusermanagement\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2013 
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 * @package datec_feusermanagement
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class FeUserController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {
	
	/**
	 * feUserGroupRepository
	 *
	 * @var \TYPO3\CMS\Extbase\Domain\Repository\FrontendUserGroupRepository
	 * @inject
	 */
	protected $feUserGroupRepository;
	
	/**
	 * feUserRepository
	 *
	 * @var \Datec\DatecFeusermanagement\Domain\Repository\FeUserRepository
	 * @inject
	 */
	protected $feUserRepository;
		
	/**
	 * $extKey
	 */
	protected $extKey;
	
	/**
	 * $feUserId
	 */
	protected $feUserId;
	
	/**
	 * passwordValidator
	 *
	 * @var \Datec\DatecFeusermanagement\Validation\Validator\PasswordValidator
	 * @inject
	 */
	protected $passwordValidator;
	
	/**
	 * $mailservice
	 * 
	 * @var \Datec\DatecFeusermanagement\Services\MailService
	 * @inject
	 */
	protected $mailService;
	
	/**
	 * $lang
	 */
	protected $lang;
	
	/**
	 * Initialize action method validators
	 *
	 * @return void
	 */
	protected function initializeActionMethodValidators() {
		parent::initializeActionMethodValidators();
			
		foreach ($this->arguments as $argument) {
			// is create action for jobentry?
			if ($argument->getName() == 'feUser' && ($this->actionMethodName == 'createAction' || $this->actionMethodName == 'pwUpdateAction')) {
				// get validator for model
				$validator = $argument->getValidator();
					
				// add custom zipcode validator
				$passwordValidator = $this->passwordValidator;
				$validator->addValidator($passwordValidator);
			}
		}
	}
	
	/**
	 *
	 * @param \TYPO3\CMS\Extbase\Domain\Repository\FrontendUserRepository $frontendUserRepository
	 */
	public function injectFrontendUserRepository(\TYPO3\CMS\Extbase\Domain\Repository\FrontendUserRepository $frontendUserRepository) {
		$this->frontendUserRepository = $frontendUserRepository;
	}
	
	/**
	 * initialize current action
	 * 
	 * @return void
	 */
	public function initializeAction() {
		$this->extKey = $this->request->getControllerExtensionKey();
		$this->lang = $_GLOBAL['TSFE']->lang;
				
		$this->configurationErrors = $this->checkConfiguration();

		$this->feUserId = $this->getFEUserId();
	}
	
	/**
	 * action actionOverview
	 * 
	 * 
	 * @return void
	 */
	public function actionOverviewAction() {		
		if (count($this->configurationErrors) > 0) {
			$this->forward('stop', 'FeUser');
		}
		
		// is the user on the 'myData' Page?
		if($GLOBALS['TSFE']->id == $this->settings['myDataPid']) {
			$this->redirect('myData', 'FeUser', 'datecfeusermanagement', $this->request->getArguments());
		}
		$this->view->assign("lang", $this->lang);
		// user is logged in?
		if($this->feUserId) {
			$myDataPid = $this->settings['myDataPid'];
			$this->view->assign('myDataPid', $myDataPid);
		}	
				
	}
	
	/**
	 * action registerForm
	 * 
	 * @param \Datec\DatecFeusermanagement\Domain\Model\FeUser $feUser
	 * @dontvalidate $feUser
	 * 
	 * @return void
	 */
	public function registerFormAction(\Datec\DatecFeusermanagement\Domain\Model\FeUser $feUser = NULL) {
	
		// supress technnical error messages
		// will probably be handled differently with Typo3 6.3
		$msgs = $this->getControllerContext()->getFlashMessageQueue()->getAllMessagesAndFlush();
		foreach($msgs as $msg) {
			//if ($msg->getMessage() != 'An error occurred while trying to call Datec\DatecFeusermanagement\Controller\FeUserController->createAction()') {
				$this->addFlashMessage($msg->getMessage(), $msg->getTitle(), $msg->getSeverity());
			//}
		}		
		$this->view->assign("lang", $this->lang);
		$this->view->assign('feUser', $feUser);
		$this->view->assign('settings', $this->settings);
	}
	
	/**
	 * action create
	 * 
	 * @param \Datec\DatecFeusermanagement\Domain\Model\FeUser $feUser
	 * @ignorevalidation $feUser
	 *
	 * @return void
	 */
	public function createAction(\Datec\DatecFeusermanagement\Domain\Model\FeUser $feUser) {	
		$success = FALSE;
		
		$username = $feUser->getUsername();
		$firstName = $feUser->getFirstName();
		$lastName = $feUser->getLastName();
		$email = $feUser->getEmail();
		$password = $feUser->getPassword();
		
		if (!empty($username) && !empty($firstName) && !empty($lastName) && !empty($email) && !empty($password)) {
			$knownUser = $this->feUserRepository->findUserByUsernameOrEmail($username,$email);		
			if ($knownUser == NULL) {
				$userGroups = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
				$userGroups->attach($this->feUserGroupRepository->findByUid($this->settings['notRegistredFeUsergroup']));
				$validationHash = md5(\TYPO3\CMS\Core\Utility\GeneralUtility::generateRandomBytes(64));
	            $randHashDB = md5($validationHash);         
		
				$feUser->setUsergroup($userGroups);
				$feUser->setTxDatecFeusermanagementHashvalue($randHashDB);
				$feUser->setPid($this->settings['storagePid']);
				
				$this->feUserRepository->add($feUser);
				
				$validationLink = $this->getValidationUri($validationHash, 'validateRegistration');
				$msg = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_mail_messages.greeting',$this->extKey) .LF.
						$firstName.' '.$lastName.','.LF.LF.
						\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_mail_register.validationMail_01',$this->extKey).LF.
						$validationLink.LF.
						\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_mail_register.validationMail_02',$this->extKey).LF.
						\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_mail_messages.closing',$this->extKey);
				$subject = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_mail_subjects.registerMail',$this->extKey);
				
				$sendMail = $this->mailService->sendExternMail($subject, $msg, $email, $firstName.' '.$lastName, $this->settings);			
				if ($sendMail) {
					$success = TRUE;
				} else {
					$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.sendExternMailError',$this->extKey));
				}			
			} else {
				$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.createKnownUserError',$this->extKey));
			}
		} else {
			$this->redirect('registerForm', 'FeUser', 'datecfeusermanagement', $this->request->getArguments());
		}		
		
		$this->view->assign('feUser', $feUser);
		$this->view->assign('success', $success);
		$this->view->assign('settings', $this->settings);
	}
	
	/**
	 * action validateRegistration
	 *
	 * @return void
	 */
	public function validateRegistrationAction() {
		if($this->request->hasArgument('tx_datec_feusermanagement_hashvalue')){
			$randHashDB = md5($this->request->getArgument('tx_datec_feusermanagement_hashvalue'));
			
			$validFeUser = $this->feUserRepository->findUserByValidationHash($randHashDB, NULL);
			if($validFeUser != NULL) {				
				$userGroups = $this->objectManager->get('TYPO3\CMS\Extbase\Persistence\ObjectStorage');
				$userGroups->attach($this->feUserGroupRepository->findByUid($this->settings['defaultFeUsergroup']));
				
				$plainPassword = $validFeUser->getPassword();
				$saltedPassword = $this->saltPassword($plainPassword); // try salted password
				if (!empty($saltedPassword)) {
					$validFeUser->setPassword($saltedPassword);
				} else {
					$validFeUser->setPassword($plainPassword);
				}				
				
				$validFeUser->setUsergroup($userGroups);
				$validFeUser->setTxDatecFeusermanagementHashvalue('');
				
				$this->feUserRepository->update($validFeUser);
				
				// login this user and redirect to start
				$this->loginFeUser($validFeUser->getUsername(), $plainPassword);
				$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.validateRegistrationSuccess',$this->extKey));
				$this->redirect('actionOverview', 'FeUser', 'datecfeusermanagement');
			} else {
				$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.wrongValidationHashError',$this->extKey));
			}
		} else {
			$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.noValidationHashError',$this->extKey));
		}
		
	}
	
	/**
	 * action pwResetRequestForm
	 *
	 * @return void
	 */
	public function pwResetRequestFormAction() {			
		$success = FALSE;
		
		if ($this->request->hasArgument('pwResetRequest') || $this->request->hasArgument('email')) {
			$feUser = $this->feUserRepository->findUserByEmail($this->request->getArgument('email'));
			if($feUser != NULL) {
				$validationHash = md5(\TYPO3\CMS\Core\Utility\GeneralUtility::generateRandomBytes(64));
				$timestamp = time() + 604800; // 24hrs
				$randHashDB = md5($validationHash);
				$hashTime = $timestamp.'/'.$validationHash;
				$hashTimeDB = $timestamp.'/'.$randHashDB;
				
				$feUser->setTxDatecFeusermanagementHashvalue($hashTimeDB);
				
				$this->feUserRepository->update($feUser);
					
				$validationLink = $this->getValidationUri($hashTime, 'pwResetForm');
				$msg = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_mail_messages.greeting',$this->extKey) .LF.
						$feUser->getFirstName().' '.$feUser->getLastName().','.LF.LF.
						\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_mail_pwreset.validationMail_01',$this->extKey).LF.
						$validationLink.LF.
						\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_mail_pwreset.validationMail_02',$this->extKey).LF.
						\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_mail_messages.closing',$this->extKey);
				$subject = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_mail_subjects.pwResetMail',$this->extKey);
					
				$sendMail = $this->mailService->sendExternMail($subject, $msg, $feUser->getEmail(), $feUser->getFirstName().' '.$feUser->getLastName(), $this->settings);
				if ($sendMail) {
					$success = TRUE;
				} else {
					$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.sendExternMailError',$this->extKey));
				}
				
				$this->view->assign('email', $this->request->getArgument('email'));
				$this->view->assign('pwResetRequest', TRUE);
			} else {
				$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.noUserForEmailError',$this->extKey));
			}
		}

		$this->view->assign('success', $success);
		$this->view->assign('settings', $this->settings);
			
	}
	
	/**
	 * action pwResetForm
	 *
	 * @return void
	 */
	public function pwResetFormAction() {		
		if($this->request->hasArgument('tx_datec_feusermanagement_hashvalue')){
			$hashTime = explode('/',$this->request->getArgument('tx_datec_feusermanagement_hashvalue'),2);
			$timestamp = $hashTime[0];
			$randHashDB = md5($hashTime[1]);
			
			$validFeUser = $this->feUserRepository->findUserByValidationHash($randHashDB, $timestamp);
			if($validFeUser != NULL) {
				$validFeUser->setPassword('');
				
				$this->view->assign('feUser', $validFeUser);
				$this->flashMessageContainer->flush(); // aviod error info messages when displaying validation erros
			} else {
				$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.wrongValidationHashError',$this->extKey));
			}
		} else {
			$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.noValidationHashError',$this->extKey));
		}

		$this->view->assign('settings', $this->settings);
		$this->view->assign("lang", $this->lang);
				
	}
	
	/**
	 * action pwUpdate
	 *
	 * @param \Datec\DatecFeusermanagement\Domain\Model\FeUser $feUser
	 *
	 * @return void
	 */
	public function pwUpdateAction(\Datec\DatecFeusermanagement\Domain\Model\FeUser $feUser) {	
		$plainPassword = $feUser->getPassword();
		$saltedPassword = $this->saltPassword($plainPassword); // try salted password
		if (!empty($saltedPassword)) {
			$feUser->setPassword($saltedPassword);
		} else {
			$feUser->setPassword($plainPassword);
		}
		$feUser->setTxDatecFeusermanagementHashvalue('');
		
		$this->feUserRepository->update($feUser);
		
		// login user and redirect
		$this->loginFeUser($feUser->getUsername(), $plainPassword);
		$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.pwUpdateSuccess',$this->extKey));		
		$this->redirect('actionOverview', 'FeUser', 'datecfeusermanagement', $this->args);
	}
	
	/**
	 * action myData
	 *
	 * displays user's personal information
	 *
	 * @return void
	 */
	public function myDataAction() {
		if ($this->feUserId) {
			$feUser = $this->feUserRepository->findByUid($this->feUserId);
			
			$this->view->assign('feUser', $feUser);			
		} else {
			$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.myDataError',$this->extKey));
		}
		
		$this->view->assign('settings', $this->settings);
	}
	
	/**
	 * action stop
	 * 
	 * displays errors on page
	 *
	 * @return void
	 */
	public function stopAction() {
		foreach ($this->configurationErrors as $configurationErrorMsg) {
			$this->addFlashMessage($configurationErrorMsg);
		}
	}
	
	/**
	 * check plugin configuration
	 *
	 * @return array $errors
	 */
	private function checkConfiguration() {
		$errors = array();
		if (!isset($this->settings['defaultFeUsergroup']) || empty($this->settings['defaultFeUsergroup'])) {
			$errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.defaultFeUsergroupError',$this->extKey);
		}
		if (!isset($this->settings['feUsermanagementPid']) || empty($this->settings['feUsermanagementPid'])) {
			$errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.feUsermanagementPidError',$this->extKey);
		}
		if (!isset($this->settings['returnPagePid']) || empty($this->settings['returnPagePid'])) {
			$errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.returnPagePidError',$this->extKey);
		}
		if (!isset($this->settings['myDataPid']) || empty($this->settings['myDataPid'])) {
			$errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.myDataPidError',$this->extKey);
		}
		if (!isset($this->settings['storagePid']) || empty($this->settings['storagePid'])) {
			$errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecfeusermanagement_view_messages.storagePidError',$this->extKey);
		}
		
		return $errors;
	}
	
	/**
	 * get current fe user and retrieve his Id
	 *
	 * @return array $feUserId or boolean FALSE if no allowed user found
	 */
	private function getFEUserId() {
		if(isset($GLOBALS['TSFE']->fe_user->user[uid])) {
			$feUserId = $GLOBALS['TSFE']->fe_user->user[uid];
			return $feUserId;
		}
	
		return FALSE;
	}
	
	/**
	 * Get the URI for a validation action.
	 * 
	 * @param string $validationHash
	 * @param string $targetAction
	 *
	 * @return string the URI
	 */
	private function getValidationUri($validationHash, $targetAction) {	
		$uriBuilder = $this->controllerContext->getUriBuilder();
		$argumentPrefix = 'tx_datecfeusermanagement_feuser';
		if ($this->request->hasArgument('controller')) {
			$arguments['controller'] = $this->request->getArgument('controller');
		}
		if ($this->request->hasArgument('action')) {
			$arguments['action'] = $targetAction;
		}		
		if ($this->request->hasArgument('format') && $this->arguments['format'] !== '') {
			$arguments['format'] = $this->request->getArgument('format');
		}
		$arguments['tx_datec_feusermanagement_hashvalue'] = $validationHash;

		return $uriBuilder
			->reset()
			->setArguments(array($argumentPrefix => $arguments))
			->setAddQueryString(TRUE)
			->setArgumentsToBeExcludedFromQueryString(array($argumentPrefix, 'cHash'))
			->setFormat($arguments['format'])
			->setCreateAbsoluteUri(TRUE)
			->build();
	}
	
	/**
	 * Salt a password using 'saltedpasswords' extension
	 *
	 * @param string $password A new plain-text password
	 *
	 * @return string the salted password
	 */
	private function saltPassword($password) {
		$saltedPassword = '';
		
		if (\TYPO3\CMS\Core\Utility\ExtensionManagementUtility::isLoaded('saltedpasswords')) {
			if (\TYPO3\CMS\Saltedpasswords\Utility\SaltedPasswordsUtility::isUsageEnabled('FE')) {
				$objSalt = \TYPO3\CMS\Saltedpasswords\Salt\SaltFactory::getSaltingInstance(NULL);
				if (is_object($objSalt)) {
					$saltedPassword = $objSalt->getHashedPassword($password);
				}
			}
		}
		
		return $saltedPassword;
	}
	
	/**
	 * creates a new user session on login information
	 * uses salted passwords authentification service
	 *
	 * @param string $password
	 * @param string $password
	 *
	 */	
	private function loginFeUser($username, $password) {
		$loginData = array(
				'uname' => $username,
				'uident' => $password,
				'uident_text' => $password,
				'status' => 'login'
		);
		
		$GLOBALS['TSFE']->fe_user->checkPid = 0;
		$info = $GLOBALS['TSFE']->fe_user->getAuthInfoArray();
		$user = $GLOBALS['TSFE']->fe_user->fetchUserRecord( $info['db_user'], $loginData['uname'] );
		if ( \TYPO3\CMS\Saltedpasswords\SaltedPasswordService::compareUident($user,$loginData) ) {
			$GLOBALS["TSFE"]->fe_user->user = $GLOBALS["TSFE"]->fe_user->fetchUserSession();
			$GLOBALS['TSFE']->loginUser = 1;
			$GLOBALS['TSFE']->fe_user->fetchGroupData();
			$GLOBALS['TSFE']->fe_user->start();
			$GLOBALS["TSFE"]->fe_user->createUserSession($user);
			$GLOBALS["TSFE"]->fe_user->loginSessionStarted = TRUE;
		}
	}
	
}
?>