<?php
namespace Datec\DatecFeusermanagement\Domain\Repository;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2013 
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package datec_feusermanagement
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class FeUserRepository extends \TYPO3\CMS\Extbase\Domain\Repository\FrontendUserRepository {
	
	/**
	 * Finds an feUser object matching the given username.
	 *
	 * @param mixed $identifier The identifier of the object to find
	 * @return object The matching object if found, otherwise NULL
	 * @api
	 */
	public function findUserByUsernameOrEmail($username,$email) {
		$contraints = array();
		$query = $this->createQuery();
		$query->getQuerySettings()->setRespectStoragePage(FALSE);
		$query->getQuerySettings()->setRespectSysLanguage(FALSE);
		$query->getQuerySettings()->setIgnoreEnableFields(TRUE);
			
		$contraints[] = $query->equals('username', $username);
		$contraints[] = $query->equals('email', $email);
		
		$object = $query->matching($query->logicalOr($contraints))->execute()->getFirst();
	
		return $object;
	}
	
	/**
	 * Finds an feUser object matching the given hash.
	 *
	 * @param string $randHashDB The hash on wich to identify one user
	 * @param tstamp $timestamp (optional) Timestamp for hash valid time
	 * 
	 * @return object The matching object if found, otherwise NULL
	 * @api
	 */
	public function findUserByValidationHash($randHashDB, $timestamp = NULL) {
		$query = $this->createQuery();
		$query->getQuerySettings()->setRespectStoragePage(FALSE);
		$query->getQuerySettings()->setRespectSysLanguage(FALSE);
		$query->getQuerySettings()->setIgnoreEnableFields(TRUE);
		
		if (isset($timestamp)) {
			$hashDB = $timestamp.'/'.$randHashDB;
		} else {
			$hashDB = $randHashDB;
		}
	
		$object = $query->matching($query->equals('tx_datec_feusermanagement_hashvalue', $hashDB))->execute()->getFirst();
	
		return $object;
	}
	
	/**
	 * Finds an feUser object matching the given email.
	 *
	 * @param string $email The email on wich to identify one user
	 * @return object The matching object if found, otherwise NULL
	 * @api
	 */
	public function findUserByEmail($email) {
		$query = $this->createQuery();
		$query->getQuerySettings()->setRespectStoragePage(FALSE);
		$query->getQuerySettings()->setRespectSysLanguage(FALSE);
		$query->getQuerySettings()->setIgnoreEnableFields(TRUE);
	
		$object = $query->matching($query->equals('email', $email))->execute()->getFirst();
	
		return $object;
	}
}
?>